@@ -29,7 +29,11 @@ cloudCheck <- function(toDigest, checksumsFileID = NULL, cloudFolderID = NULL) {
   suppressMessages(checksums <- cloudDownloadChecksums(checksumsFileID))
   hashExists <- checksums$cacheId == dig
   out <- if (isTRUE(any(hashExists))) {
-    cloudDownload(checksums[hashExists])
+    if (sum(hashExists) > 1) {
+      stop("running cloudCheck with >1 object toDigest is not implemented yet;",
+           "contact eliot.mcintire@canada.ca")
+    }
+    cloudDownload(checksums[hashExists])[[1]]
   } else {
     NULL
   }
@@ -104,7 +108,11 @@ cloudWrite <- function(object, digest, cloudFolderID = NULL, checksums, checksum
 #' @importFrom googledrive as_id drive_download
 #' @rdname cloudExtras
 #' @seealso \code{\link{cloudSyncCache}}, \code{\link{cloudCache}}, \code{\link{cloudExtras}}
-cloudDownloadChecksums <- function(checksumsFileID) {
+cloudDownloadChecksums <- function(checksumsFileID = NULL, cloudFolderID = NULL) {
+
+  if (is.null(checksumsFileID))
+    checksumsFileID <- getChecksumsFileID(cloudFolderID)
+
   checksumsFilename <- tempfile(fileext = ".rds");
   os <- tolower(Sys.info()[["sysname"]])
   .onLinux <- .Platform$OS.type == "unix" && unname(os) == "linux" &&
@@ -116,6 +124,7 @@ cloudDownloadChecksums <- function(checksumsFileID) {
         bb <- future::value(.reproEnv$futureEnv)       # but if not, then force its value here
   drive_download(as_id(checksumsFileID), path = checksumsFilename, verbose = FALSE)
   checksums <- readRDS(checksumsFilename)
+  return(checksums)
 }
 
 #' @details
@@ -182,11 +191,24 @@ cloudUpdateChecksums <- function(checksums, checksumsFileID) {
 #' # now both local and cloud exist
 #' a4 <- cloudCache(rnorm, 1, cloudFolderID = newDir$id)
 #'
-#' # Clean up -- see cloudSyncCache
+#' #  more than one cacheRepo
+#' opts <- options("reproducible.cachePath" = c(tempdir(), file.path(tempdir(), "test"),
+#'                                              file.path(tempdir(), "test2")),
+#'                 "reproducible.ask" = FALSE)
+#' cachePaths <- getOption("reproducible.cachePath")
+#' Cache(rnorm, 4, cacheRepo = cachePaths[3]) # put it in 3rd cacheRepo
+#'
+#' # gets it locally even though it is in the 3rd cacheRepo, uploads to cloudCache
+#' cloudCache(rnorm, 4, cloudFolderID = newDir$id)
+#'
+#' # Clean up -- also see cloudSyncCache
+#' clearCache(ask = FALSE)
+#' # lapply(cachePaths, clearCache, ask = FALSE)
+#' cloudSyncCache(cloudFolderID = newDir$id)
+#'
 #' }
 cloudCache <- function(..., useCloud = getOption("reproducible.useCloud", TRUE),
                        checksumsFileID = NULL, cloudFolderID = NULL) {
-
   hasCopy <- FALSE
   hasCloudCopy <- FALSE
 
@@ -197,10 +219,10 @@ cloudCache <- function(..., useCloud = getOption("reproducible.useCloud", TRUE),
       repo <- checkPath(repo, create = TRUE)
     })
   }
-  cacheRepo <- cacheRepos[[1]]
+  # cacheRepo <- cacheRepos[[1]]
 
   #cacheRepo <- suppressMessages(.checkCacheRepo(list(...), create = TRUE))#fnDetails$modifiedDots, create = TRUE))
-  suppressMessages(archivist::createLocalRepo(cacheRepo))
+  suppressMessages(archivist::createLocalRepo(cacheRepos[1]))
   if (isTRUE(useCloud)) {
 
     if (is.null(checksumsFileID))
@@ -213,8 +235,13 @@ cloudCache <- function(..., useCloud = getOption("reproducible.useCloud", TRUE),
       dots <- .CacheDigestWithOmitArgs(sys.call(), envir = sys.frame(-1))
     }
     dig <- CacheDigest(dots)
-    checkLocalFirst <- try(suppressMessages(showCache(userTags = dig$outputHash, x = cacheRepo)))
-    hasLocalCopy <- NROW(checkLocalFirst) > 0 && !is(checkLocalFirst, "try-error")
+    suppressMessages(checkLocalFirst <- lapply(cacheRepos, function(cacheRepo, ...) {
+      tryCatch(showCache(x = cacheRepo, userTags = dig$outputHash), error = function(x) NULL)
+    }))
+
+    #checkLocalFirst <- try(suppressMessages(showCache(userTags = dig$outputHash, x = cacheRepo)))
+    hasLocalCopy <- NROW(rbindlist(checkLocalFirst, fill = TRUE, use.names = TRUE)) > 0 &&
+      !is(checkLocalFirst, "try-error")
     if (hasLocalCopy) {
       if (is.null(checksumsFileID))
         checksumsFileID <- getChecksumsFileID(cloudFolderID)
@@ -280,7 +307,7 @@ cloudCache <- function(..., useCloud = getOption("reproducible.useCloud", TRUE),
     if ("cacheRepo" %in% names(list(...))) {
       Cache(...)
     } else {
-      Cache(..., cacheRepo = cacheRepo)
+      Cache(..., cacheRepo = cacheRepos)
     }
   } else {
     cachedCopy$object
@@ -357,9 +384,13 @@ getChecksumsFileID <- function(cloudFolderID) {
 #' @examples
 #' \dontrun{
 #'   #make a google drive folder
-#'   opts <- options("reproducible.cachePath" = tempdir(), "reproducible.ask" = FALSE)
+#'   #   Can use >1 cacheRepo
+#'   opts <- options("reproducible.cachePath" = c(tempdir()),
+#'                   "reproducible.ask" = FALSE)
+#'   cachePaths <- getOption("reproducible.cachePath")
 #'   library(googledrive)
 #'   newDir <- drive_mkdir("testFolder")
+#'   #a <- Cache(rnorm, 1, cacheRepo = getOption("reproducible.cachePath")[3])
 #'   a <- Cache(rnorm, 1)
 #'   b <- Cache(rnorm, 2)
 #'
@@ -373,12 +404,14 @@ getChecksumsFileID <- function(cloudFolderID) {
 #'   cloudSyncCache(cloudFolderID = newDir$id)
 #'
 #'   # clean up
-#'   clearCache(ask = FALSE)
+#'   lapply(cachePaths, clearCache, ask = FALSE)
+#'   # clearCache(ask = FALSE) # if there were only 1 cacheRepo
 #'   cloudSyncCache(cloudFolderID = newDir$id)
 #'
 #'   #######################################################################
 #'   # use showCache args to have control ... on upload & delete NOTE difference!
 #'   #######################################################################
+#'   # a <- Cache(rnorm, 1, cacheRepo = getOption("reproducible.cachePath")[3]) # multiple cacheRepos!
 #'   a <- Cache(rnorm, 1)
 #'   b <- Cache(rnorm, 2)
 #'   # only sync the one with rnorm, 2 as arguments
@@ -425,20 +458,26 @@ getChecksumsFileID <- function(cloudFolderID) {
 #'   cloudSyncCache(cloudFolderID = newDir$id, cacheIds = tag6)
 #'
 #'   # clean up
-#'   clearCache(ask = FALSE)
+#'   # clearCache(ask = FALSE) # if only one cacheRepo
+#'   lapply(cachePaths, clearCache, ask = FALSE)
 #'   cloudSyncCache(cloudFolderID = newDir$id)
 #'
 #'   # To remove whole folder:
 #'   drive_rm(as_id(newDir$id))
 #'   options(opts)
 #' }
-cloudSyncCache <- function(cacheRepo = getOption("reproducible.cachePath")[1],
+cloudSyncCache <- function(cacheRepo = getOption("reproducible.cachePath"),
                            checksumsFileID = NULL, cloudFolderID = NULL,
                            delete = TRUE, upload = TRUE, download = !delete,
                            ask = getOption("reproducible.ask"),
                            cacheIds = NULL,
                            ...) {
-  if (isTRUE(download)) delete <- FALSE
+  if (isTRUE(download)) {
+    if (isTRUE(delete)) {
+      delete <- FALSE
+      message("download is TRUE; setting delete to FALSE")
+    }
+  }
   if (is.null(checksumsFileID)) {
     if (!is.null(cloudFolderID)) {
       checksumsFileID <- getChecksumsFileID(cloudFolderID)
@@ -448,59 +487,94 @@ cloudSyncCache <- function(cacheRepo = getOption("reproducible.cachePath")[1],
   }
 
   suppressMessages(checksums <- cloudDownloadChecksums(checksumsFileID))
-  suppressMessages(localCache <- showCache(x = cacheRepo, ...))
+
+  # get cacheRepo if not supplied
+  names(cacheRepo) <- normPath(cacheRepo)
+  cacheRepos <- lapply(cacheRepo, function(repo) {
+      repo <- checkPath(repo, create = TRUE)
+    })
+
+  # cacheRepo <- cacheRepos[[1]]
+  suppressMessages(localCaches <- lapply(cacheRepos, ..., function(cacheRepo, ...) {
+    tryCatch(showCache(x = cacheRepo, ...), error = function(x) NULL)
+  }))
+  #suppressMessages(localCache <- showCache(x = cacheRepo, ...))
   out <- NULL
 
   needToDelete <- NULL
-  uniqueCacheArtifacts <- unique(localCache$artifact)
-  if (!is.null(uniqueCacheArtifacts))
-    cacheIDs <- localCache[artifact %in% uniqueCacheArtifacts & tagKey == "cacheId"]$tagValue
+  uniqueCacheArtifacts <- lapply(localCaches, function(x) unique(x$artifact))
+  cacheIDs <- lapply(cacheRepos, function(uca) {
+    if (!is.null(uniqueCacheArtifacts[[uca]]))
+      localCaches[[uca]][artifact %in% uniqueCacheArtifacts[[uca]] & tagKey == "cacheId"]$tagValue
+
+  })
 
   if (!is.null(cacheIds)) {
-    cacheIdsExisting <- which(cacheIDs %in% cacheIds)
-    if (length(cacheIdsExisting) == 0) {
+    cacheIDs <- lapply(cacheIDs, function(cacheIDsInner) {
+      cacheIdsExisting <- which(cacheIDsInner %in% cacheIds)
+      cacheIDsInner[cacheIdsExisting]
+    })
+    if (all(unlist(lapply(cacheIDs, function(x) length(x) == 0)))) {
       message("None of the supplied cacheIds exist locally: \n  ", paste(cacheIds, sep = "\n  "))
     }
-    cacheIDs <- cacheIDs[cacheIdsExisting]
+
   }
 
-  if (NROW(localCache) && isTRUE(upload)) {
-    needToUpload <- !(cacheIDs %in% checksums$cacheId)
-
-    if (sum(needToUpload) > 0) {
-      message(crayon::blue("Uploading", sum(needToUpload), "files, ",
-                           "\n ", paste(cacheIDs[needToUpload], ".rda ( == ",
-                                        uniqueCacheArtifacts[needToUpload],
-                                        ".rda in local cache)", sep = "",
-                                        collapse = "\n  ")))
-      out <- lapply(uniqueCacheArtifacts[needToUpload], function(art) {
-        cacheID <- localCache[artifact == art & tagKey == "cacheId"]$tagValue
-        drive_upload(file.path(cacheRepo, "gallery", paste0(art, ".rda")),
-                     path = as_id(cloudFolderID),
-                     name = paste0(cacheID, ".rda"),
-                     verbose = FALSE)
-      })
-      checksums <- rbindlist(
-        list(checksums,
-             data.table(
-               cacheId = cacheIDs[needToUpload],
-               id = rbindlist(out)$id,
-               time = as.character(Sys.time()),
-               filesize = file.size(file.path(cacheRepo, "gallery",
-                                              paste0(uniqueCacheArtifacts[needToUpload], ".rda")))
-             )
-        ),
-        use.names = TRUE, fill = TRUE)
-      out <- rbindlist(out)
+  ########################################################
+  # Upload to the Cloud
+  ########################################################
+  if ( any(unlist(lapply(localCaches, NROW))>0) ) {
+    needToUpload <- !(unlist(cacheIDs) %in% checksums$cacheId)
+    if (isTRUE(upload)) {
+      if (sum(needToUpload) > 0) {
+        df <- data.frame(repo = rep(names(uniqueCacheArtifacts), sapply(uniqueCacheArtifacts, length)),
+                   artifact = unname(unlist(uniqueCacheArtifacts)), stringsAsFactors = FALSE)
+        message(crayon::blue("Uploading", sum(needToUpload), "files, ",
+                             "\n ", paste(unlist(cacheIDs)[needToUpload], ".rda ( == ",
+                                          unlist(uniqueCacheArtifacts)[needToUpload],
+                                          ".rda in local cache)", sep = "",
+                                          collapse = "\n  ")))
+        whCache <- match(df$repo, unlist(cacheRepos ))
+        uniqueOnes <- df[needToUpload,]
+        out <- lapply(seq(NROW(uniqueOnes)), function(artSeq) {
+          repo <- uniqueOnes[artSeq, "repo"]
+          art <- uniqueOnes[artSeq, "artifact"]
+          cacheID <- localCaches[[repo]][artifact == art & tagKey == "cacheId"]$tagValue
+          drive_upload(file.path(repo, "gallery", paste0(art, ".rda")),
+                       path = as_id(cloudFolderID),
+                       name = paste0(cacheID, ".rda"),
+                       verbose = FALSE)
+        })
+        checksums <- rbindlist(
+          list(checksums,
+               data.table(
+                 cacheId = unlist(cacheIDs)[needToUpload],
+                 id = rbindlist(out)$id,
+                 time = as.character(Sys.time()),
+                 filesize = file.size(file.path(df[needToUpload, "repo"], "gallery",
+                                                paste0(df[needToUpload, "artifact"], ".rda")))
+               )
+          ),
+          use.names = TRUE, fill = TRUE)
+        out <- rbindlist(out)
+      }
+    } else {
+      if (length(needToUpload) > 0)
+        message("Need to upload, but upload = FALSE: \n  ",
+                paste(unlist(cacheIDs)[needToUpload]))
     }
 
   } else {
-    if (NROW(localCache) == 0) {
+    if (NROW(rbindlist(localCaches, use.names = TRUE, fill = TRUE)) == 0) {
       needToDelete <- rep(TRUE, NROW(checksums))
     }
   }
+
+  ########################################################
+  # Delete in the Cloud
+  ########################################################
   if (is.null(needToDelete)) {
-    if (!is.null(uniqueCacheArtifacts)) {
+    if (!is.null(unlist(uniqueCacheArtifacts))) {
       needToDelete <- if (!is.null(cacheIds)) {
         out <- (checksums$cacheId %in% cacheIds)
         if (all(!out)) {
@@ -508,13 +582,13 @@ cloudSyncCache <- function(cacheRepo = getOption("reproducible.cachePath")[1],
         }
         out
       } else {
-        !(checksums$cacheId %in% cacheIDs)
+        !(checksums$cacheId %in% unlist(cacheIDs))
       }
 
     }
   }
 
-  if (isTRUE(delete) && any(needToDelete)) {
+  if (isTRUE(delete) && any(needToDelete)) { # there are 1 or more to delete, and delete is TRUE
     out <- if (isTRUE(ask)) {
       readline(paste0("Are you sure you want to delete:\n  ",
                       paste(checksums$cacheId[needToDelete], ".rda",
@@ -535,12 +609,17 @@ cloudSyncCache <- function(cacheRepo = getOption("reproducible.cachePath")[1],
     }
   } else {
     if (any(needToDelete)) {
-      needToDownload <- needToDelete
+      needToDownload <- needToDelete # sync when local is missing -- either delete the cloud or download from cloud
       if (isTRUE(download)) {
-        if (!cacheIds %in% localCache[artifact %in% uniqueCacheArtifacts & tagKey == "cacheId"]$tagValue) {
-          out <- cloudDownload(checksums[needToDownload,])
+        message("Object in cloud, not in local; delete = FALSE, download = TRUE; downloading...: \n  ",
+                paste(checksums$cacheId[needToDownload], ".rda",
+                      sep = "", collapse = "\n  "))
+        cacheIdsToDownload <- checksums$cacheId[needToDownload]
+        outs <- cloudDownload(checksums[needToDownload,])
+        suppressMessages(cacheRepo <- .checkCacheRepo(cacheRepos[1], create = TRUE)) # write to 1st one
+        message("  Writing to local cacheRepo")
+        saved <- lapply(outs, function(out) {
           objSize <- sum(unlist(objSize(out)))
-          #preDigestUnlistTrunc <- unlist(.unlistToCharacter(CacheDigest(list(out))$preDigest, 3))
           userTags <- c(#if (!is.na(fnDetails$functionName))
             #paste0("function:", fnDetails$functionName),
             paste0("object.size:", objSize),
@@ -550,9 +629,7 @@ cloudSyncCache <- function(cacheRepo = getOption("reproducible.cachePath")[1],
           )
 
           written <- 0
-          suppressMessages(cacheRepo <- .checkCacheRepo(cacheRepo, create = TRUE))
           setattr(out, "tags", paste0("cacheId:", checksums[needToDownload, cacheId]))
-          message("  Writing to local cacheRepo")
           while (written >= 0) {
             saved <- suppressWarnings(try(silent = TRUE,
                                           archivist::saveToLocalRepo(
@@ -576,20 +653,19 @@ cloudSyncCache <- function(cacheRepo = getOption("reproducible.cachePath")[1],
               -1
             }
           }
-        } else {
-          message("  cacheIds already present locally")
-        }
+
+        })
       } else {
         message("There were files identified to delete to create full sync, but delete = FALSE")
+        #message("  cacheIds already present locally")
       }
     }
-
   }
+  message("uploading updated checksums file on googledrive")
   suppressMessages(cloudUpdateChecksums(checksums, checksumsFileID))
   return(invisible(checksums))
 }
 
-
 cloudUploadFileAndChecksums <- function(objectFile, cloudFolderID, digest,
                                         checksums, checksumsFileID) {
   uploadRes <- try(drive_upload(objectFile, path = as_id(cloudFolderID),
@@ -622,17 +698,16 @@ cloudUploadFileAndChecksums <- function(objectFile, cloudFolderID, digest,
 }
 
 cloudDownload <- function(checksums) {
-  if (NROW(checksums) > 1) {
-    stop("Can only download 1 file at a time, currently")
-  }
-  objectFilename2 <- tempfile(fileext = ".rda");
-  a <- checksums$filesize
-  class(a) <- "object_size"
-  message("  downloading object from google drive; this could take a while; it is: ",
-          format(a, "auto"))
-  drive_download(as_id(checksums[, id]), path = objectFilename2, verbose = FALSE)
-  env <- new.env()
-  load(objectFilename2, envir = env)
-  as.list(env)[[1]]
+  out <- lapply(seq(NROW(checksums)), function(checksumsIndex) {
+    objectFilename2 <- tempfile(fileext = ".rda");
+    a <- checksums[checksumsIndex,]$filesize
+    class(a) <- "object_size"
+    message("  downloading object from google drive; it is: ",
+            format(a, "auto"))
+    drive_download(as_id(checksums[checksumsIndex, id]), path = objectFilename2, verbose = FALSE)
+    env <- new.env()
+    load(objectFilename2, envir = env)
+    as.list(env)[[1]]
+  })
 
 }
