@@ -33,6 +33,9 @@
 convertPaths <- function(x, patterns, replacements) {
   stopifnot(is(x, "character"))
   stopifnot(length(patterns) == length(replacements))
+  patterns <- normPath(patterns)
+  replacements <- normPath(replacements)
+  x = normPath(x)
   for (i in seq_along(patterns)) {
     x <- gsub(x = x, pattern = patterns[i], replacement = replacements[i])
   }
@@ -51,7 +54,7 @@ convertRasterPaths <- function(x, patterns, replacements) {
       if (length(x) > 1) {
         x <- lapply(x, convertRasterPaths, patterns, replacements)
       } else {
-        x <- raster()
+        x <- raster(x)
       }
     }
 
