@@ -112,7 +112,7 @@ reproducibleOptions <- function() {
     reproducible.ask = TRUE,
     reproducible.cloudChecksumsFilename = file.path(dirname(.reproducibleTempCacheDir),
                                                     "checksums.rds"),
-    reproducible.cachePath = file.path(.reproducibleTempCacheDir),
+    reproducible.cachePath = normPath(file.path(.reproducibleTempCacheDir)),
     reproducible.destinationPath = NULL,
     reproducible.futurePlan = FALSE, #future::plan("multiprocess"), #memoise
     reproducible.inputPaths = NULL,
